local result

function love.draw()
    if result then
        love.graphics.draw(result)
    end
end

local base = 64
local size = 512

function love.filedropped(file)
	file:open "r"
    local source = love.image.newImageData(love.filesystem.newFileData(file:read(), file:getFilename()))
    local colors = {}

    for x=0, source:getWidth()-1 do
        for y=0, source:getHeight()-1 do
            local r, g, b = source:getPixel(x,y)
            local hash = ("%d\t%d\t%d"):format(r*255,g*255,b*255)
            if not colors[hash] then
                colors[hash] = {r,g,b}
            end
        end
    end

    result = love.image.newImageData(size,size)
    for r=0, base-1 do
        for g=0, base-1 do
            for b=0, base-1 do
                local bestDist = math.huge
                local bestColor
                local h1, s1, v1 = r/(base-1), g/(base-1), b/(base-1)
                for _, color in pairs(colors) do
                    local h2, s2, v2 = unpack(color)
                    local dist = (h1 - h2)^2 + 5*(s1 - s2)^2 + (v1 - v2)^2
                    if dist < bestDist then
                        bestColor = color
                        bestDist = dist
                    end
                end

                local i = r + g*base + b*base*base
                local x, y = math.floor(i/size), i%size
                result:setPixel(x, y, unpack(bestColor))
            end
        end
    end

    result:encode("png", "palette.png")
    result = love.graphics.newImage(result)
end
